@@ -18,7 +18,7 @@ level =    'cloud', 'cloudspace', 'site'
 args =
     --id               cloud/cloudspace ID
     --ids              comma separated list of cloud/cloudspace IDs
-    --load             load data from the previous run
+    --load             loads data from the previous run
     --skip             List of cloudspace IDs to skip acting upon
     --slackchannel     Slack channel for status posting
     --slackurl         Slack webhook URL
@@ -58,6 +58,7 @@ import traceback
 
 from .cons3rtapi import Cons3rtApi, Cons3rtApiError
 from .cons3rtcli import validate_ids
+from .cons3rtconfig import get_report_dir
 from .logify import Logify
 from .slack import SlackAttachment, SlackMessage
 
@@ -65,11 +66,8 @@ from .slack import SlackAttachment, SlackMessage
 # Set up logger name for this module
 mod_logger = Logify.get_name() + '.remoteaccesscontroller'
 
-# Report directory
-report_dir = os.path.join(os.path.expanduser('~'), 'cons3rt_reports')
-
 # Output File
-out_file = os.path.join(report_dir, 'remote_access_data.csv')
+out_file = os.path.join(get_report_dir(), 'remote_access_data.csv')
 
 valid_commands = ['enable', 'disable', 'lock', 'print', 'toggle', 'unlock']
 valid_commands_str = ','.join(valid_commands)
@@ -99,7 +97,9 @@ class EnabledStatus(object):
 
 class EnabledRemoteAccess(object):
     def __init__(self, cloud_id, cloud_name, cloud_type, cloudspace_id, cloudspace_name, active_status,
-                 enabled=None, run_id=None, run_name=None, instance_type=None, locked_status=None):
+                 enabled=None, run_id=None, run_name=None, instance_type=None, locked_status=None,
+                 remote_access_external_ip=None, remote_access_internal_ip=None, remote_access_port=None,
+                 rdp_proxy_enabled=None, template_name=None, user_count=None):
         self.cloud_id = cloud_id
         self.cloud_name = cloud_name
         self.cloud_type = cloud_type
@@ -111,11 +111,17 @@ class EnabledRemoteAccess(object):
         self.run_name = run_name
         self.instance_type = instance_type
         self.locked_status = locked_status
+        self.remote_access_external_ip = remote_access_external_ip
+        self.remote_access_internal_ip = remote_access_internal_ip
+        self.remote_access_port = remote_access_port
+        self.rdp_proxy_enabled = rdp_proxy_enabled
+        self.template_name = template_name
+        self.user_count = user_count
 
     @staticmethod
     def header_row():
         return 'CloudId,CloudName,CloudType,CloudspaceId,CloudspaceName,ActiveStatus,EnabledStatus,RaRunId,RaRunName,' \
-               'Size,LockedStatus'
+               'Size,LockedStatus,ExternalIp,InternalIp,Port,RdpProxyEnabled,TemplateName,UserCount'
 
     def __str__(self):
         out_str = ''
@@ -160,7 +166,31 @@ class EnabledRemoteAccess(object):
         else:
             out_str += 'None,'
         if self.locked_status:
-            out_str += 'ENABLED'
+            out_str += 'ENABLED' + ','
+        else:
+            out_str += 'None,'
+        if self.remote_access_external_ip:
+            out_str += self.remote_access_external_ip + ','
+        else:
+            out_str += 'None'
+        if self.remote_access_internal_ip:
+            out_str += self.remote_access_internal_ip + ','
+        else:
+            out_str += 'None'
+        if self.remote_access_port:
+            out_str += str(self.remote_access_port) + ','
+        else:
+            out_str += 'None,'
+        if self.rdp_proxy_enabled:
+            out_str += str(self.rdp_proxy_enabled) + ','
+        else:
+            out_str += 'None,'
+        if self.template_name:
+            out_str += str(self.template_name) + ','
+        else:
+            out_str += 'None,'
+        if self.user_count:
+            out_str += str(self.user_count)
         else:
             out_str += 'None'
         return out_str
@@ -203,7 +233,7 @@ class RemoteAccessController(object):
         self.remote_access_run_info = []
 
     def send_slack(self, msg, color='good'):
-        """Sends slack message if URL and channel were provided
+        """Sends Slack message if URL and channel were provided
 
         :param msg: (str) message to send
         :param color: (str) color of the attachment
@@ -226,8 +256,8 @@ class RemoteAccessController(object):
                     {
                         'state': 'ACTIVE',
                         'cloudType': 'VCloudCloud',
-                        'id': 7,
-                        'name': 'Hanscom Cloud'
+                        'id': 1,
+                        'name': 'arc1'
                     }
                 ]
 
@@ -319,16 +349,84 @@ class RemoteAccessController(object):
                 log.info('Found cloudspace data already for cloudspace: {i}'.format(i=str(cloudspace['id'])))
                 continue
 
+            # Get the cloudspace data
+            cloudspace_id = 'NotFound'
+            cloudspace_name = 'NotFound'
+            cloudspace_state = 'NotFound'
+            if 'id' in cloudspace.keys():
+                cloudspace_id = cloudspace['id']
+            if 'name' in cloudspace.keys():
+                cloudspace_name = cloudspace['name']
+            if 'state' in cloudspace.keys():
+                cloudspace_state = cloudspace['state']
+
+            # Get the cloud data
+            cloud_id = 'NotSpecified'
+            cloud_name = 'NotSpecified'
+            cloud_type = 'NotSpecified'
+            if 'cloud' in cloudspace.keys():
+                if 'id' in cloudspace['cloud'].keys():
+                    cloud_id = cloudspace['cloud']['id']
+                if 'name' in cloudspace['cloud'].keys():
+                    cloud_name = cloudspace['cloud']['name']
+                if 'cloudType' in cloudspace['cloud'].keys():
+                    cloud_type = cloudspace['cloud']['cloudType']
+
+            # Get the boundary IP
+            boundary_ip = None
+            if 'accessPoint' in cloudspace.keys():
+                boundary_ip = cloudspace['accessPoint']
+            else:
+                if 'networks' in cloudspace.keys():
+                    for network in cloudspace['networks']:
+                        if 'networkFunction' in network.keys():
+                            if network['networkFunction'] == 'CONS3RT':
+                                if 'boundaryIpAddress' in network.keys():
+                                    boundary_ip = network['boundaryIpAddress']
+
+            # Print the boundary IP
+            if boundary_ip:
+                log.info('Using cloudspace boundary IP address: {i}'.format(i=boundary_ip))
+
+            # Get the remote access config data
+            remote_access_ip = 'NotSpecified'
+            remote_access_port = 'NotSpecified'
+            rdp_proxy_enabled = 'NotSpecified'
+            template_name = 'NotSpecified'
+            if 'remoteAccessConfig' in cloudspace.keys():
+                if 'remoteAccessIpAddress' in cloudspace['remoteAccessConfig'].keys():
+                    remote_access_ip = cloudspace['remoteAccessConfig']['remoteAccessIpAddress']
+                if 'remoteAccessPort' in cloudspace['remoteAccessConfig'].keys():
+                    remote_access_port = cloudspace['remoteAccessConfig']['remoteAccessPort']
+                if 'rdpProxyClientEnabled' in cloudspace['remoteAccessConfig'].keys():
+                    rdp_proxy_enabled = cloudspace['remoteAccessConfig']['rdpProxyClientEnabled']
+                if 'templateName' in cloudspace['remoteAccessConfig'].keys():
+                    template_name = cloudspace['remoteAccessConfig']['templateName']
+
+            # Get the user count for the cloudspace
+            try:
+                cloudspace_users = self.c5t.list_users_in_virtualization_realm(vr_id=cloudspace_id)
+            except Cons3rtApiError as exc:
+                msg = 'Problem listing users for cloudspace ID: {i}'.format(i=str(cloudspace_id))
+                raise RemoteAccessControllerError(msg) from exc
+            cloudspace_user_count = len(cloudspace_users)
+
             # Add cloudspace data
             log.info('Adding cloudspace data for cloudspace ID: {i}'.format(i=str(cloudspace['id'])))
             self.remote_access_run_info.append(
                 EnabledRemoteAccess(
-                    cloud_id=cloudspace['cloud']['id'],
-                    cloud_name=cloudspace['cloud']['name'],
-                    cloud_type=cloudspace['cloud']['cloudType'],
-                    cloudspace_id=cloudspace['id'],
-                    cloudspace_name=cloudspace['name'],
-                    active_status=cloudspace['state']
+                    cloud_id=cloud_id,
+                    cloud_name=cloud_name,
+                    cloud_type=cloud_type,
+                    cloudspace_id=cloudspace_id,
+                    cloudspace_name=cloudspace_name,
+                    active_status=cloudspace_state,
+                    remote_access_external_ip=boundary_ip,
+                    remote_access_internal_ip=remote_access_ip,
+                    remote_access_port=remote_access_port,
+                    rdp_proxy_enabled=rdp_proxy_enabled,
+                    template_name=template_name,
+                    user_count=cloudspace_user_count
                 )
             )
         log.info('Found {n} total cloudspaces'.format(n=str(len(self.remote_access_run_info))))
@@ -426,12 +524,12 @@ class RemoteAccessController(object):
         self.get_cloudspace_data()
 
         # RA data will be queried for VRs with this state
-        active_statii = ['ACTIVE', 'ENTERING_MAINTENANCE', 'MAINTENANCE']
+        active_status = ['ACTIVE', 'ENTERING_MAINTENANCE', 'MAINTENANCE']
 
         for ra_data in self.remote_access_run_info:
 
             # Skip if the cloudspace is not active
-            if ra_data.active_status not in active_statii:
+            if ra_data.active_status not in active_status:
                 log.info('Skipping RA data retrieval for cloudspace ID {i} with state: {s}'.format(
                     i=str(ra_data.cloudspace_id), s=ra_data.active_status))
                 RemoteAccessController.append_remote_access_run(ra_run_data=ra_data)
@@ -905,8 +1003,9 @@ class RemoteAccessController(object):
         log = logging.getLogger(self.cls_logger + '.read_remote_access_run_data_from_file')
 
         if not self.load_data:
-            log.info('--load not specified, removing existing data file')
-            os.remove(out_file)
+            if os.path.isfile(out_file):
+                log.info('--load not specified, removing existing data file')
+                os.remove(out_file)
 
         if not os.path.isfile(out_file):
             log.info('No remote access run data exists to read, creating with header...')
@@ -1004,7 +1103,7 @@ def main():
     if args.slackurl:
         slack_url = args.slackurl
 
-    # Parse the unlock arg
+    # Parse the "unlock" arg
     unlock = False
     if args.unlock:
         unlock = True
