@@ -11,6 +11,7 @@ import os
 import yaml
 
 from pycons3rt3.cons3rtapi import Cons3rtApi
+from pycons3rt3.cons3rtconfig import get_data_dir, get_report_dir
 from pycons3rt3.exceptions import Cons3rtApiError, Cons3rtReportsError
 from pycons3rt3.logify import Logify
 
@@ -21,11 +22,8 @@ __author__ = 'Joe Yennaco'
 # Set up logger name for this module
 mod_logger = Logify.get_name() + '.reports'
 
-# Report directory
-report_dir = os.path.join(os.path.expanduser('~'), 'cons3rt_reports')
-
 # Data files
-cons3rt_data_file = os.path.join(report_dir, 'cons3rt_tmp_data.yml')
+cons3rt_data_file = os.path.join(get_data_dir(), 'cons3rt_tmp_data.yml')
 
 
 def generate_team_report(team_id, cons3rt_api, load=False):
@@ -38,15 +36,11 @@ def generate_team_report(team_id, cons3rt_api, load=False):
     :raises: Cons3rtReportsError
     """
     log = logging.getLogger(mod_logger + '.generate_team_report')
-    try:
-        team_id = int(team_id)
-    except ValueError:
-        msg = 'Team ID provided is not a valid integer: {t}'.format(t=team_id)
-        raise Cons3rtReportsError(msg)
+    validate_team_id(team_id)
     log.info('Generating team VM tally for team ID: {t}'.format(t=str(team_id)))
 
-    if not os.path.isdir(report_dir):
-        os.makedirs(report_dir, exist_ok=True)
+    if not os.path.isdir(get_report_dir()):
+        os.makedirs(get_report_dir(), exist_ok=True)
 
     if load:
         cons3rt_vm_data = read_cons3rt_data()
@@ -65,26 +59,21 @@ def generate_team_report(team_id, cons3rt_api, load=False):
     return cons3rt_vm_data
 
 
-def generate_team_asset_report(team_id, cons3rt_api):
+def generate_team_asset_report(team_id):
     """Generates reports for the specified team ID
 
     :param team_id: (int) ID of the team
-    :param cons3rt_api: (Cons3rtApi) object
     :return: None
     :raises: Cons3rtReportsError
     """
     log = logging.getLogger(mod_logger + '.generate_team_asset_report')
-    try:
-        team_id = int(team_id)
-    except ValueError:
-        msg = 'Team ID provided is not a valid integer: {t}'.format(t=team_id)
-        raise Cons3rtReportsError(msg)
+    validate_team_id(team_id)
     log.info('Generating team VM tally for team ID: {t}'.format(t=str(team_id)))
 
-    if not os.path.isdir(report_dir):
-        os.makedirs(report_dir, exist_ok=True)
+    if not os.path.isdir(get_report_dir()):
+        os.makedirs(get_report_dir(), exist_ok=True)
 
-    asset_data = generate_team_asset_list(team_id=team_id, cons3rt_api=cons3rt_api)
+    asset_data = generate_team_asset_list(team_id=team_id)
 
     # Generate the output
     generate_asset_output(team_id=team_id, asset_data=asset_data)
@@ -100,21 +89,7 @@ def generate_cons3rt_data(team_id, cons3rt_api):
     """
     log = logging.getLogger(mod_logger + '.generate_cons3rt_data')
 
-    log.info('Retrieving team info for team ID: {i}'.format(i=str(team_id)))
-    try:
-        team_details = cons3rt_api.get_team_details(team_id=team_id)
-    except Cons3rtApiError as exc:
-        msg = 'Problem retrieving team details for team ID: {i}'.format(i=str(team_id))
-        raise Cons3rtReportsError(msg) from exc
-    team_name = team_details['name']
-    log.info('Retrieved details on team [{n}] with ID: {i}'.format(n=team_name, i=str(team_id)))
-
-    log.info('Retrieving deployment run and host details for team ID: {i}'.format(i=str(team_id)))
-    try:
-        run_host_list = cons3rt_api.list_host_details_in_team(team_id=team_id)
-    except Cons3rtApiError as exc:
-        msg = 'Problem getting a list of runs and hosts in team ID: {i}'.format(i=str(team_id))
-        raise Cons3rtReportsError(msg) from exc
+    run_host_list, team_name = get_run_host_list(team_id, cons3rt_api)
 
     # Get data from output
     log.info('Generating output from DR host...')
@@ -150,12 +125,22 @@ def generate_cons3rt_data(team_id, cons3rt_api):
                     )
             else:
                 log.warning('installations data not found in run host ID: {i}'.format(i=str(drh_details['id'])))
+
+            # Get the system role name
             system_role_str = ''
             if 'systemRole' in drh_details.keys():
                 system_role_str = drh_details['systemRole']
+
+            # Get the template name
             template_name = ''
             if 'physicalMachineOrTemplateName' in drh_details.keys():
                 template_name = drh_details['physicalMachineOrTemplateName']
+
+            # Get the snapshot date if available, or put N/A
+            snapshot_date_str = 'NA'
+            if 'snapshotDate' in drh_details.keys():
+                snapshot_date_str = drh_details['snapshotDate']
+
             drh_data.append(
                 {
                     'team_id': team_id,
@@ -176,6 +161,9 @@ def generate_cons3rt_data(team_id, cons3rt_api):
                     'cpus': drh_details['numCpus'],
                     'ram_mb': drh_details['ram'],
                     'storage_gb': storage_gb,
+                    'snapshot_available': drh_details['snapshotAvailable'],
+                    'snapshot_date': snapshot_date_str,
+                    'snapshot_storage_gb': storage_gb if drh_details['snapshotAvailable'] else 0,
                     'networks': network_str,
                     'assets': assets_str
                 }
@@ -184,13 +172,12 @@ def generate_cons3rt_data(team_id, cons3rt_api):
     return drh_data
 
 
-def generate_team_asset_list(team_id, cons3rt_api):
+def generate_team_asset_list(team_id):
     """Returns software and container asset data for the specified team ID
 
     Assets used in currently active deployment runs.
 
     :param team_id: (int) ID of the team
-    :param cons3rt_api: (Cons3rtApi) object
     :return: (list) of assets used by the team:
     [
       {
@@ -202,21 +189,7 @@ def generate_team_asset_list(team_id, cons3rt_api):
     log = logging.getLogger(mod_logger + '.generate_team_asset_list')
     cons3rt_api = Cons3rtApi()
 
-    log.info('Retrieving team info for team ID: {i}'.format(i=str(team_id)))
-    try:
-        team_details = cons3rt_api.get_team_details(team_id=team_id)
-    except Cons3rtApiError as exc:
-        msg = 'Problem retrieving team details for team ID: {i}'.format(i=str(team_id))
-        raise Cons3rtReportsError(msg) from exc
-    team_name = team_details['name']
-    log.info('Retrieved details on team [{n}] with ID: {i}'.format(n=team_name, i=str(team_id)))
-
-    log.info('Retrieving deployment run and host details for team ID: {i}'.format(i=str(team_id)))
-    try:
-        run_host_list = cons3rt_api.list_host_details_in_team(team_id=team_id)
-    except Cons3rtApiError as exc:
-        msg = 'Problem getting a list of runs and hosts in team ID: {i}'.format(i=str(team_id))
-        raise Cons3rtReportsError(msg) from exc
+    run_host_list, team_name = get_run_host_list(team_id, cons3rt_api)
 
     # Generate the asset list
     asset_list = []
@@ -258,6 +231,9 @@ def generate_cons3rt_header():
            'Cpus,' \
            'RamMb,' \
            'StorageGb,' \
+           'Snapshot,' \
+           'SnapshotDate,' \
+           'SnapshotStorageGb,' \
            'Networks,' \
            'Assets'
 
@@ -285,6 +261,9 @@ def generate_cons3rt_row(cons3rt_vm):
            str(cons3rt_vm['cpus']) + ',' + \
            str(cons3rt_vm['ram_mb']) + ',' + \
            str(cons3rt_vm['storage_gb']) + ',' + \
+           str(cons3rt_vm['snapshot_available']) + ',' + \
+           str(cons3rt_vm['snapshot_date']) + ',' + \
+           str(cons3rt_vm['snapshot_storage_gb']) + ',' + \
            cons3rt_vm['networks'] + ',' + \
            cons3rt_vm['assets']
 
@@ -310,31 +289,74 @@ def read_cons3rt_data():
 
 
 def generate_cons3rt_output(team_id, cons3rt_data):
-    log = logging.getLogger(mod_logger + '.generate_cons3rt_output')
     csv = str(generate_cons3rt_header()) + '\n'
     for cons3rt_vm in cons3rt_data:
         csv += generate_cons3rt_row(cons3rt_vm=cons3rt_vm) + '\n'
     report_time = datetime.datetime.now()
     report_timestamp = report_time.strftime('%Y%m%d-%H%M%S')
     output_file_name = 'team_{t}_data_{s}.csv'.format(t=str(team_id), s=str(report_timestamp))
-    output_file_path = os.path.join(report_dir, output_file_name)
-    if os.path.isfile(output_file_path):
-        log.info('Removing existing output file: {f}'.format(f=output_file_path))
-        os.remove(output_file_path)
-    log.info('Generating output file: {f}'.format(f=output_file_path))
-    with open(output_file_path, 'w') as f:
-        f.write(csv)
+    write_output_file(output_file_name, csv)
 
 
 def generate_asset_output(team_id, asset_data):
-    log = logging.getLogger(mod_logger + '.generate_asset_output')
     csv = str(generate_asset_header()) + '\n'
     for asset_dict in asset_data:
         csv += generate_asset_row(asset_dict=asset_dict) + '\n'
     report_time = datetime.datetime.now()
     report_timestamp = report_time.strftime('%Y%m%d-%H%M%S')
     output_file_name = 'team_{t}_asset_data_{s}.csv'.format(t=str(team_id), s=str(report_timestamp))
-    output_file_path = os.path.join(report_dir, output_file_name)
+    write_output_file(output_file_name, csv)
+
+
+def validate_team_id(team_id):
+    """Validates the team ID
+
+    :param team_id: (int) ID of the team
+    :return: None
+    :raises: Cons3rtReportsError
+    """
+    try:
+        team_id = int(team_id)
+    except ValueError:
+        msg = 'Team ID provided is not a valid integer: {t}'.format(t=team_id)
+        raise Cons3rtReportsError(msg)
+
+
+def get_run_host_list(team_id, cons3rt_api):
+    """
+
+    :param team_id: (int) ID of the team
+    :param cons3rt_api: (Cons3rtApi) object
+    :return: (tuple) list of run hosts in the team, and str team name
+    :raises: Cons3rtReportsError
+    """
+    log = logging.getLogger(mod_logger + '.get_run_host_list')
+    log.info('Retrieving team info for team ID: {i}'.format(i=str(team_id)))
+    try:
+        team_details = cons3rt_api.get_team_details(team_id=team_id)
+    except Cons3rtApiError as exc:
+        msg = 'Problem retrieving team details for team ID: {i}'.format(i=str(team_id))
+        raise Cons3rtReportsError(msg) from exc
+    team_name = team_details['name']
+    log.info('Retrieved details on team [{n}] with ID: {i}'.format(n=team_name, i=str(team_id)))
+
+    log.info('Retrieving deployment run and host details for team ID: {i}'.format(i=str(team_id)))
+    try:
+        run_host_list = cons3rt_api.list_host_details_in_team(team_id=team_id)
+    except Cons3rtApiError as exc:
+        msg = 'Problem getting a list of runs and hosts in team ID: {i}'.format(i=str(team_id))
+        raise Cons3rtReportsError(msg) from exc
+    return run_host_list, team_name
+
+
+def write_output_file(output_file_name, csv):
+    """Write the report files
+
+    :param output_file_name: (str) output file name
+    :param csv: (str) comma-separated values
+    """
+    log = logging.getLogger(mod_logger + '.write_output_file')
+    output_file_path = os.path.join(get_report_dir(), output_file_name)
     if os.path.isfile(output_file_path):
         log.info('Removing existing output file: {f}'.format(f=output_file_path))
         os.remove(output_file_path)
